
 * prevent export of all the non-public APIs
 * make a few things static that failed to link with Xcode 5 after hiding the
   non-public APIs.
 * Fix (I think?) pthread_key_create destructor.
 * Disable call to pthread_init() ... this is an unfortunately named private
   symbol in Mac OS X's library, and it breaks TLS when MariaDB calls it.

 I just roughed these in. These should be cleaned up and submitted upstream.

 --ryan. (icculus@icculus.org)


=== modified file 'include/mysql.h'
--- include/mysql.h	2013-10-14 12:17:54 +0000
+++ include/mysql.h	2013-11-01 04:20:43 +0000
@@ -78,6 +78,8 @@
   } MEM_ROOT;
 #endif
 
+#pragma GCC visibility push(default)
+
 extern unsigned int mysql_port;
 extern char *mysql_unix_port;
 
@@ -509,6 +511,8 @@
 #define mysql_library_init mysql_server_init
 #define mysql_library_end mysql_server_end
 
+#pragma GCC visibility pop
+
 /* new api functions */
 
 #define HAVE_MYSQL_REAL_CONNECT

=== modified file 'libmariadb/hash.c'
--- libmariadb/hash.c	2013-10-10 13:14:15 +0000
+++ libmariadb/hash.c	2013-11-01 04:16:58 +0000
@@ -94,7 +94,7 @@
   handle inline functions that are not defined as native types
 */
 
-inline char*
+inline static char*
 hash_key(HASH *hash,const uchar *record,uint *length,my_bool first)
 {
   if (hash->get_key)
@@ -194,7 +194,7 @@
 #ifndef __SUNPRO_C				/* SUNPRO can't handle this */
 inline
 #endif
-unsigned int rec_hashnr(HASH *hash,const uchar *record)
+static unsigned int rec_hashnr(HASH *hash,const uchar *record)
 {
   uint length;
   uchar *key= (uchar*) hash_key(hash,record,&length,0);

=== modified file 'libmariadb/my_init.c'
--- libmariadb/my_init.c	2013-08-14 14:08:21 +0000
+++ libmariadb/my_init.c	2013-11-01 14:35:42 +0000
@@ -69,7 +69,7 @@
   my_init_done=1;
 #ifdef THREAD
 #if defined(HAVE_PTHREAD_INIT)
-  pthread_init();			/* Must be called before DBUG_ENTER */
+//  pthread_init();			/* Must be called before DBUG_ENTER */
 #endif
   my_thread_global_init();
 #ifndef _WIN32

=== modified file 'libmariadb/my_thr_init.c'
--- libmariadb/my_thr_init.c	2013-10-20 07:14:48 +0000
+++ libmariadb/my_thr_init.c	2013-11-01 14:35:24 +0000
@@ -50,9 +50,17 @@
 	the function my_thread_global_free must be called from
 	somewhere before final exit of the library */
 
+static void free_THR_KEY_mysys(void *ptr)
+{
+  /* pthread_key_create destructors must set the pointer to NULL,
+     or the system will destruct the same pointer multiple times. */
+  free(ptr);
+  pthread_setspecific(THR_KEY_mysys, NULL);
+}
+
 my_bool my_thread_global_init(void)
 {
-  if (pthread_key_create(&THR_KEY_mysys,free))
+  if (pthread_key_create(&THR_KEY_mysys,free_THR_KEY_mysys))
   {
     fprintf(stderr,"Can't initialize threads: error %d\n",errno);
     exit(1);

